#IMPORTING LIBRARIES
import subprocess
import numpy as np
import pandas as pd
import pickle
from Bio import SeqIO
import re
import os
import csv
import sys
import sklearn
import random
import glob


global job_id
global out
global dir


# read job_id as last variable

job_id = int(sys.argv[4])

print("JOB ID: ",job_id,type(job_id))


dir = '/gpsr/webserver/cgidocs/tmp/thppred/thppred_{}'.format(job_id)


if not os.path.exists(dir):
    # Create the directory
    os.makedirs(dir)

# Define positive and negative motif lists
positive_motif_list = ['VSW', 'CSV', 'NWY', 'PSVF', 'KGQP', 'CgapSV', 'WgapQG', 'VTgapC', 'YgapCgapN',
                       'WgapYgapD', 'YaromaticC', 'neutralKNneutral', 'Qaliphaticaromaticneutralaliphatic',
                       'basicVneutralN', 'sulfurbasicgapSneutral', 'CbasicgapSneutral', 'YgaparomaticC',
                       'SgapWneutralneutral', 'cyclicneutralCpolarmedium', 'cyclicacyclicCpolarmedium',
                       'neutralVpolarWsurface', 'surfacecycliclargeCcharged', 'aliphaticacyclicVpolarW',
                       'cyclicgapCchargedmedium', 'cyclicneutralCpolarmedium', 'aromaticgapCchargedmedium']

negative_motif_list = ['ILDL', 'FPDW', 'GFPV', 'LDLW', 'PMgapT', 'ILgapDL', 'PVacidicP', 'TPaliphaticPaliphatic',
                       'aliphaticPaliphaticbasicP', 'LacidicLaromatic', 'LacidicLaromaticaliphatic',
                       'PVgapacidicP', 'ValiphaticFgapP', 'PVchargedP', 'PVchargedPsurface', 'PVchargedPacyclic',
                       'PVchargedPpolar', 'PVchargedPneutral', 'PVgapchargedP', 'PVchargedgapP', 'PVchargedP']

motifs_all = positive_motif_list+negative_motif_list



#########################################################################################################################################################################
#Prediction Module


# Dictionary to represent motif groups
groups_keys = ['aliphatic', 'aromatic', 'charged', 'basic', 'acidic', 'polar', 'neutral', 'cyclic', 
               'acyclic', 'large', 'medium', 'surface', 'hydrophobic', 'sulfur', 'gap']

groups = {
    'aliphatic': '[IVLAG]',
    'aromatic': '[FWY]',
    'charged': '[DEHKR]',
    'basic': '[KRH]',
    'acidic': '[DE]',
    'polar': '[RNDCEQHKSI]',
    'neutral': '[STNQ]',
    'cyclic': '[P]',
    'acyclic': '[ARNDCEQGILKMSTV]',
    'large': '[REQHILKMFWY]',
    'medium': '[NDCPTV]',
    'surface': '[RNDEQGHKPSTY]',
    'hydrophobic': '[AGILMFPWYV]',
    'sulfur': '[CM]',
    'gap': '[ACDEFGHIKLMPQRSTUVWY]'
}

def generate_motif_patterns(motif):
    motif_patterns = []
    for group, regex in groups.items():
        motif = re.sub(group, regex, motif)
    motif_patterns.append(motif)
    return motif_patterns

def find_motif(sequence):
    lst = []
    for motif in positive_motif_list:
        motif_patterns = generate_motif_patterns(motif)
        for motif_pattern in motif_patterns:
            if re.search(motif_pattern, sequence):
                lst.append(1)
            
    for motif in negative_motif_list:
        motif_patterns = generate_motif_patterns(motif)
        for motif_pattern in motif_patterns:
            if re.search(motif_pattern, sequence):
                lst.append(0)
    
    if 1 in lst and 0 not in lst:
        return 1
    elif 0 in lst and 1 not in lst:
        return 0
    else:
        return "-"

def read_fasta(file_path):
    sequences = []
    sequence_ids = []
    with open(file_path, 'r') as file:
        sequence = ''
        seq_id = ''
        for line in file:
            if line.startswith('>'):
                if sequence:
                    sequences.append(sequence)
                    sequence_ids.append(seq_id)
                    sequence = ''
                seq_id = line.strip()
            else:
                sequence += line.strip()
        if sequence:
            sequences.append(sequence)
            sequence_ids.append(seq_id)
    return sequence_ids, sequences


def run_pfeature_comp(input_fasta):
    
    try:
        output = dir+'/'+'pfeature_result_{}.csv'.format(job_id)
        subprocess.run([ '/gpsr/software/pfeature/comp/dist/pfeature_comp', '-i', input_fasta, '-o', output, '-j', 'AAC,BTC,DDR,CeTD,PAAC,APAAC,SOC'], check=True)
        print("pfeature_comp.py executed successfully.")
    except subprocess.CalledProcessError as e:
        print("Error: pfeature_comp.py returned non-zero exit status {}.".format(e.returncode))
    except Exception as e:
        print("An error occurred: {}".format(str(e)))

def extract_svcl1_features():
    f = ["AAC_I", "AAC_K", "AAC_L", "AAC_R","BTC_T", "BTC_H", "BTC_S", "BTC_D","DDR_A", "DDR_C", "DDR_D", "DDR_E", "DDR_H", "DDR_I", "DDR_M", "DDR_N", "DDR_P", "DDR_Q", "DDR_R", "DDR_S", "DDR_T", "DDR_V", "DDR_W", "DDR_Y","CeTD_HB1", "CeTD_VW3", "CeTD_SS3", "CeTD_SA2", "CeTD_11_VW", "CeTD_11_CH","CeTD_12_VW", "CeTD_12_SS", "CeTD_12_SA", "CeTD_13_PZ", "CeTD_21_SA",
    "CeTD_22_CH", "CeTD_23_SA", "CeTD_31_PZ", "CeTD_31_CH", "CeTD_32_HB","CeTD_32_VW", "CeTD_32_CH", "CeTD_32_SS", "CeTD_33_HB", "CeTD_33_CH","CeTD_33_SA", "CeTD_100_p_CH1", "CeTD_100_p_PZ2", "CeTD_100_p_HB3","CeTD_75_p_PZ3", "CeTD_100_p_PZ3", "PAAC1_I", "PAAC1_K", "PAAC1_L", "PAAC1_R","APAAC1_I", "APAAC1_K", "APAAC1_L", "APAAC1_R","SOC1_G1"]
    
    df = pd.read_csv(dir+'/'+'pfeature_result_{}.csv'.format(job_id))
    df = df.filter(items=f)
    print("ssssssssssssssssssssssssssssssssssssssssssssssss: ",df)
    df.to_csv(dir+'/'+'extracted_columns_{}.csv'.format(job_id), index=False)

    
    # Check if the file exists
    if os.path.isfile(dir + '/' + 'extracted_columns_{}.csv'.format(job_id)):
        print("File generated successfully!")
    else:
        print("File not found.")

def run_model():
    df_loaded = pd.read_csv(dir + '/' +'extracted_columns_{}.csv'.format(job_id))

    with open('/usr1/webserver/cgibin/thppred/rf_model1.pkl', 'rb') as model_file:
        model = pickle.load(model_file)

    predictions = model.predict(df_loaded)
    scores = model.predict_proba(df_loaded)[:, 1]  # Assuming binary classification, score for positive class
    return predictions, scores

def compare_labels(predicted_labels, motif_labels):
    final_labels = []
    for pred, motif in zip(predicted_labels, motif_labels):
        if motif == '-':
            final_labels.append(pred)
        else:
            final_labels.append(motif)
    return final_labels

def main_model(input_fasta,thr):
    if not os.path.isfile(input_fasta):
        print("Error: Input fasta file '{}' not found.".format(input_fasta))
        sys.exit(1)

    run_pfeature_comp(input_fasta)
    extract_svcl1_features()
    
    predicted_labels, ml_scores = run_model()

    sequence_ids, sequences = read_fasta(input_fasta)
    #motif_labels = [find_motif(seq) for seq in sequences]

    #final_labels = compare_labels(predicted_labels, motif_labels)

    results = []
    for seq_id, pred_label, ml_score in zip(sequence_ids, predicted_labels, ml_scores):
        #motif_score = 1 if motif_label == 1 else -1 if motif_label == 0 else 0
        #combined_score = ml_score + motif_score
        prediction = "Therapeutic" if ml_score > thr else "Non-Therapeutic"
        results.append([seq_id, ml_score, prediction])

    df_results = pd.DataFrame(results, columns=["Sequence id", "ML Score", "Prediction"])
    return df_results


def motif_model(input_fasta,thr):
    if not os.path.isfile(input_fasta):
        print("Error: Input fasta file '{}' not found.".format(input_fasta))
        sys.exit(1)

    run_pfeature_comp(input_fasta)
    extract_svcl1_features()
    
    predicted_labels, ml_scores = run_model()

    sequence_ids, sequences = read_fasta(input_fasta)
    motif_labels = [find_motif(seq) for seq in sequences]

    final_labels = compare_labels(predicted_labels, motif_labels)

    results = []
    for seq_id, pred_label, motif_label, final_label, ml_score in zip(sequence_ids, predicted_labels, motif_labels, final_labels, ml_scores):
        motif_score = 0.5 if motif_label == 1 else -0.5 if motif_label == 0 else 0
        hybrid_score = ml_score + motif_score
        prediction = "Therapeutic" if hybrid_score > thr else "Non-Therapeutic"
        results.append([seq_id, ml_score, motif_score, hybrid_score, prediction])

    df_results = pd.DataFrame(results, columns=["Sequence id", "ML Score", "Motif Score", "Hybrid Score", "Prediction"])
    return df_results

#########################################################################################################################################################################

#Motif Scan Module
def motif(input_fasta):
    motifs = motifs_all

    sequences = []
    sequence_id = ""
    x = input_fasta
    with open(x, 'r') as file:

        for line in file:
            line = line.strip()
            if line.startswith('>'):
                if sequence_id != "":
                    sequences.append((sequence_id, sequence))
                sequence_id = line[1:]
                sequence = ""
            else:
                sequence += line
        sequences.append((sequence_id, sequence))

    results = []
    for sequence_id, sequence in sequences:
        motifs_found = []
        for motif in motifs:
            if re.search(motif, sequence):
                motifs_found.append(motif)
        c = len(motifs_found)
        if c>0:
            p=1
        elif c==0:
            p=0

        if motifs_found:
            highlighted_sequence = sequence
            for motif in motifs_found:
                highlighted_sequence = re.sub(motif, "[{}]".format(motif), highlighted_sequence)
            motifs_str = ", ".join(motifs_found)
            results.append((sequence_id, motifs_str, c, highlighted_sequence,p))
        else:
            results.append((sequence_id, "No Motif Found",c, sequence,p))

    df = pd.DataFrame(results, columns=["Sequence ID", "Motifs","Hits", "Sequence","Prediction"])
    df['Prediction'] = df['Prediction'].replace({1: 'Therapeutic', 0: 'Non-Therapeutic'})
    return df
#########################################################################################################################################################################



#########################################################################################################################################################################

#Design Protein Module

def substitution(input_string):
    amino_acids =['A', 'C', 'D', 'E', 'F','G', 'H', 'I', 'K', 'L','M', 'N','Y', 'P', 'Q', 'R','S', 'T', 'U', 'V', 'W']
    strings = []
    for i in range(len(input_string)):
        for j in range(len(amino_acids)):
            if input_string[i] != amino_acids[j]:  # Add this condition to check for a change in character
                new_string = input_string[:i] + amino_acids[j] + input_string[i+1:]
                strings.append(new_string)
    return strings

def translocation(seq):
    # Convert the word to a list of characters
    seq_list = list(seq)
    l=[]
    # Perform the swap
    for i in range(len(seq_list)-1):

      seq_list[i], seq_list[i+1] = seq_list[i+1], seq_list[i]

      # Convert the list back to a string
      swapped_word = ''.join(seq_list)
      l.append(swapped_word)
      seq_list = list(seq)

    return l

def deletion(seq):
    # Convert the word to a list of characters
    word = seq
    l=[]
    # Perform the swap
    for i in range(len(seq)):

      deleted_word = word[:i] + word[i + 1:]

      # Convert the list back to a string
    
      l.append(deleted_word)
      word = seq

    return l

def insertion(seq):
    # Perform insertion using string concatenation
    amino_acids =['A', 'C', 'D', 'E', 'F','G', 'H', 'I', 'K', 'L','M', 'N','Y', 'P', 'Q', 'R','S', 'T', 'U', 'V', 'W']
    word = seq
    l=[]
    for i in range(len(seq)+1):
      for j in amino_acids:

        inserted_word = word[:i] + j + word[i:]
        l.append(inserted_word)
        word = seq

    return l

def all_seqs(query):
    i = insertion(query)
    d = deletion(query)
    s = substitution(query)
    t = translocation(query)
    all = i+d+s+t
    return(all)


def generate_motif_patterns(motif):
    motif_patterns = []
    for group, regex in groups.items():
        motif = re.sub(group, regex, motif)
    motif_patterns.append(motif)
    return motif_patterns
       
    

def check_sequences(sequences, positive_motif_list):
    c=0
    results=[]
    for motif in positive_motif_list:
        motif_patterns = generate_motif_patterns(motif)
        for sequence in sequences:
            found_motif = False
            for motif in motif_patterns:
                if motif in sequence:
                    found_motif = True
                    break
            if found_motif:
            
                results.append((sequence,motif))
                c+=1
            
    if c==0:
      results.append(("Mutation Not Possible for this sequence","N.A."))


    return results,c



def generate_fasta_file(df, col_name, output_file):
    with open(output_file, 'w') as fasta_file:
        for i, sequence in enumerate(df[col_name], start=1):
            fasta_file.write('>{0}\n'.format(i))
            fasta_file.write('{0}\n'.format(sequence))
          

def compare_strings(string1, string2):
  # Check for equal length
  if len(string1) != len(string2):
    if len(string1)<len(string2):
      print("Insertion")
      return "Insertion"
    elif len(string1)>len(string2):
      print("Deletion")
      return "Deletion"

  else:
  # Iterate through each character
    for i in range(len(string1)-1):
      
      if string1[i]!=string1[i+1] and string1[i]==string2[i+1] and string1[i+1]==string2[i]:
        print("Translocation")
        return "Translocation"
        break
       

      elif string1[i] != string2[i]:
        print("Substitution")
        return "Substitution"
        break
      

def design(x):
    df = pd.DataFrame()
    motifs = positive_motif_list
    sequences = all_seqs(x)
    orignal_seq = x
    thr = 0.5

    r,c = check_sequences(sequences, positive_motif_list)
    
    df = pd.DataFrame(r, columns=["Mutated Sequence", "Positive Motifs"])
    print(df)
    

        # Specify the name of the output FASTA file
    output_file = '/usr1/webserver/cgidocs/tmp/thppred/thppred_{}/mutated_sequences.fasta'.format(job_id)

        # Generate the FASTA file
    generate_fasta_file(df, 'Mutated Sequence', output_file)

    mut = df['Mutated Sequence'].tolist()
    mut_type =[]

    for i in range(len(mut)):
        x= compare_strings(orignal_seq,mut[i])
        mut_type.append(x)

    run_pfeature_comp(output_file)
    extract_svcl1_features()
    
    predicted_labels, ml_scores = run_model()

    sequence_ids, sequences = read_fasta(output_file)
    motif_labels = [find_motif(seq) for seq in sequences]

    final_labels = compare_labels(predicted_labels, motif_labels)

    results = []
    for seq_id, pred_label, motif_label, final_label, ml_score in zip(sequence_ids, predicted_labels, motif_labels, final_labels, ml_scores):
        motif_score = 0.5 if motif_label == 1 else -0.5 if motif_label == 0 else 0
        hybrid_score = ml_score + motif_score
        prediction = "Therapeutic" if hybrid_score > thr else "Non-Therapeutic"
        results.append([seq_id, ml_score, motif_score, hybrid_score, prediction])

    df_results = pd.DataFrame(results, columns=["Sequence id", "ML Score", "Motif Score", "Hybrid Score", "Prediction"])
    



    #df1 = run_model()
   
    merged_df = pd.concat([df, df_results], axis=1)
    merged_df.insert(1, 'Mutation Type', mut_type)
    #new_df = merged_df.drop('Subject', axis=1)
    #new_df = new_df.iloc[:, [2, 3, 0, 1]]
    new_df = merged_df.drop_duplicates()
    #new_df = new_df[new_df['Prediction'] != "Non-Therapeutic"]
    
   
    if c ==0:
        return df
    elif c>0:
        return new_df

##############################################################################################################################


#Tables
## Prediction Table
def dataframe_to_html_table(df):
    # Define CSS styles
    styles = """
        <style>
            table {
                border-collapse: collapse;
                margin: 50px auto;
                font-size: 16px;
                table-layout: fixed;
                width: 67%;
            }
            th, td {
                border: 1px solid black;
                padding: 10px;
                text-align: center;
                cursor: pointer;
                position: relative;
                width: calc(100% / """ + str(len(df.columns)) + """);
                white-space: nowrap; /* Prevent text from wrapping */
                text-overflow: ellipsis; /* Clip text with an ellipsis (...) */
                overflow: hidden; /* Hide overflowing text */
                
            }
            th::after {
                content: "";
                position: absolute;
                right: 8px;
                top: 50%;
                transform: translateY(-50%);
                width: 0;
                height: 0;
                border-style: solid;
                border-width: 6px 6px 0 6px;
                border-color: #888888 transparent transparent transparent;
                opacity: 0.4;
            }
            th.asc::after {
                border-bottom-width: 0;
                border-top-color: #000000;
                opacity: 1;
            }
            th.desc::after {
                border-top-width: 0;
                border-bottom-color: #000000;
                opacity: 1;
            }
            th:hover::after {
                opacity: 1;
            }
            th:hover {
                background-color: rgba(0, 35, 88, 0.749);
            }
            

            th {
                background-color: rgba(0, 35, 88, 0.749); /* Change the background color to blue */
                color: white; /* Change the text color to white */
            }

             /* Show clipped text on hover */
            td:hover {
            overflow: visible;
            white-space: normal;
            word-wrap: break-word;
            }

            h1 {
                text-align: center;
                color: #626060;;
                font-weight: bold;
                margin: 50px;
            }

            p{
                font-family:Tahoma, Geneva, Verdana, sans-serif;
                text-align: center;
                text-justify: inter-word;
                font-size: 20px;
                
                padding-right: 100px;
                
                padding-left:100px;
            }


            .btn {
                display: flex;
                justify-content: center;
                align-items: center;
                width: 200px;
                height: 50px;
                background-color: #4CAF50;
                color: white;
                font-size: 20px;
                text-align: center;
                text-decoration: none;
                border-radius: 25px;
                margin: 50px auto;
            }

            .text_link {
                color: red;
            }
        </style>
    """
    

    table =  "<p>This page is the output of the prediction of <b>therapeutic and non-therapeutic proteins</b> among the query sequences given by the user. The table below is a provides the details of the query proteins given as input by the user with first column displaying the sequence ID, second column providing the score given by the machine learning algorithm according to the prediction model, and the third column is providing the prediction, whether the protein is therapeutic or non-therapeutic, determined by the condition, whether the score is greater or less than the user defined threshold. In case of the hybrid methods , it also provides the motif score and hybrid score before the prediction column.</p>"
    table+= "<br><br>"
    table += "<center>Job ID: <b>{}</b>. For more information, click <a href='help.php'>Help</a>. To download results as a CSV file: <a href=/tmp//thppred/thppred_{}/results_{}.csv' class='text_link' >Click Here</a></center>".format(job_id,job_id,job_id)
    

    table += "<table id='myTable'>"
    # add header row
    table += "<tr>"
    for col in df.columns:
        table += "<th onclick='sortTable(" + str(df.columns.get_loc(col)) + ", event)'>" + str(col) + "</th>"
    table += "</tr>"
    # add data rows
    for row in df.itertuples(index=False):
        table += "<tr>"
        for val in row:
            if isinstance(val, (int, float)):
                val = float(val)
                table += "<td>" + str(round(val, 2)) + "</td>"
            else:
                table += "<td>" + str(val) + "</td>"
        table += "</tr>"
    table += "</table>"

    script = """
        <script>
            var ascending = true;

            function sortTable(columnIndex, event) {
                var table, rows, switching, i, x, y, shouldSwitch;
                table = document.getElementById("myTable");
                switching = true;
                while (switching) {
                    switching = false;
                    rows = table.rows;
                    for (i = 1; i < (rows.length - 1); i++) {
                        shouldSwitch = false;
                        x = rows[i].getElementsByTagName("TD")[columnIndex];
                        y = rows[i + 1].getElementsByTagName("TD")[columnIndex];
                        if (ascending) {
                            if (x.innerHTML.toLowerCase() > y.innerHTML.toLowerCase()) {
                                shouldSwitch = true;
                                break;
                            }
                        } else {
                            if (x.innerHTML.toLowerCase() < y.innerHTML.toLowerCase()) {
                                shouldSwitch = true;
                                break;
                            }
                        }
                    }
                    if (shouldSwitch) {
                        rows[i].parentNode.insertBefore(rows[i + 1], rows[i]);
                        switching = true;
                    }
                }
                ascending = !ascending;
                event.currentTarget.classList.toggle("asc", ascending);
                event.currentTarget.classList.toggle("desc", !ascending);
            }
        </script>
    """

    return styles + script + table


## Motif Scan Table
def motif_table(df):
    # Define CSS styles
    
    styles1 = """
        <style>
            table {
                border-collapse: collapse;
                margin: 50px auto;
                font-size: 16px;
                table-layout: fixed;
                width: 67%;
                
            }
            th, td {
                border: 1px solid black;
                padding: 10px;
                text-align: center;
                cursor: pointer;
                position: relative;
                width: calc(100% / """ + str(len(df.columns)) + """);
                white-space: nowrap; /* Prevent text from wrapping */
                text-overflow: ellipsis; /* Clip text with an ellipsis (...) */
                overflow: hidden; /* Hide overflowing text */
            }
            th::after {
                content: "";
                position: absolute;
                right: 8px;
                top: 50%;
                transform: translateY(-50%);
                width: 0;
                height: 0;
                border-style: solid;
                border-width: 6px 6px 0 6px;
                border-color: #888888 transparent transparent transparent;
                opacity: 0.4;
            }
            th.asc::after {
                border-bottom-width: 0;
                border-top-color: #000000;
                opacity: 1;
            }
            th.desc::after {
                border-top-width: 0;
                border-bottom-color: #000000;
                opacity: 1;
            }
            th:hover::after {
                opacity: 1;
            }
            th:hover {
                background-color: rgba(0, 35, 88, 0.749);
            }
            th {
                background-color: rgba(0, 35, 88, 0.749);
                color: white;
            }
             /* Show clipped text on hover */
            td:hover {
            overflow: visible;
            white-space: normal;
            word-wrap: break-word;
            }

            h1 {
                text-align: center;
                color: #626060;;
                font-weight: bold;
                margin: 50px;
            }
            p{
                font-family:Tahoma, Geneva, Verdana, sans-serif;
                text-align: center;
                text-justify: inter-word;
                font-size: 20px;
                
                padding-right: 100px;
                
                padding-left:100px;
            }


            .custom-text {
                color: #e74c3c;
            }

            .btn {
                display: flex;
                justify-content: center;
                align-items: center;
                width: 200px;
                height: 50px;
                background-color: #4CAF50;
                color: white;
                font-size: 20px;
                text-align: center;
                text-decoration: none;
                border-radius: 25px;
                margin: 50px auto;
            }

            .text_link {
                color: red;
            }
        </style>
    """
   


    table =  "<p>For the <b>Motif module</b>, the results are displayed in five columns. The  table contains the Sequence ID, List of motifs found, Hits (Number of motifs if found in the query sequence submitted by the user), the query sequence and prediction of the query sequence.</p>"

    table+= "<br><br>"
    table += "<center>Job ID: <b>{}</b>. For more information, click <a href='help.php'>Help</a>. To download results as a CSV file: <a href='/dis.php?file=/tmp/thppred/thppred_{}/results_{}.csv' class='text_link'>Click Here</a></center>".format(job_id,job_id,job_id)

        

    table += "<table id='myTable'>"
    # add header row
    table += "<tr>"
    for col in df.columns:
        table += "<th onclick='sortTable(" + str(df.columns.get_loc(col)) + ", event)'>" + str(col) + "</th>"
    table += "</tr>"
    # add data rows
    for row in df.itertuples(index=False):
        table += "<tr>"
        for val in row:
            if isinstance(val, str) and '[' in val and ']' in val:
                # Wrap text within brackets in <b> tags for bold formatting
                val = val.replace('[', '<b class = custom-text>').replace(']', '</b>')
            table += "<td>" + str(val) + "</td>"
        table += "</tr>"
    table += "</table>"
    
    script = """
        <script>
            var ascending = true;

            function sortTable(columnIndex, event) {
                var table, rows, switching, i, x, y, shouldSwitch;
                table = document.getElementById("myTable");
                switching = true;
                while (switching) {
                    switching = false;
                    rows = table.rows;
                    for (i = 1; i < (rows.length - 1); i++) {
                        shouldSwitch = false;
                        x = rows[i].getElementsByTagName("TD")[columnIndex];
                        y = rows[i + 1].getElementsByTagName("TD")[columnIndex];
                        if (ascending) {
                            if (x.innerHTML.toLowerCase() > y.innerHTML.toLowerCase()) {
                                shouldSwitch = true;
                                break;
                            }
                        } else {
                            if (x.innerHTML.toLowerCase() < y.innerHTML.toLowerCase()) {
                                shouldSwitch = true;
                                break;
                            }
                        }
                    }
                    if (shouldSwitch) {
                        rows[i].parentNode.insertBefore(rows[i + 1], rows[i]);
                        switching = true;
                    }
                }
                ascending = !ascending;
                event.currentTarget.classList.toggle("asc", ascending);
                event.currentTarget.classList.toggle("desc", !ascending);
            }
        </script>
    """

    return styles1 +script + table

## Design Protein Table
def design_table(df):
    # Define CSS styles

    styles = """
        <style>
            table {
                border-collapse: collapse;
                margin: 50px auto;
                font-size: 16px;
                table-layout: fixed;
                width: 67%;
               
            }
            th, td {
                border: 1px solid black;
                padding: 10px;
                text-align: center;
                cursor: pointer;
                position: relative;
                width: calc(100% / """ + str(len(df.columns)) + """);
                white-space: nowrap; /* Prevent text from wrapping */
                text-overflow: ellipsis; /* Clip text with an ellipsis (...) */
                overflow: hidden; /* Hide overflowing text */
            }
            th::after {
                content: "";
                position: absolute;
                right: 8px;
                top: 50%;
                transform: translateY(-50%);
                width: 0;
                height: 0;
                border-style: solid;
                border-width: 6px 6px 0 6px;
                border-color: #888888 transparent transparent transparent;
                opacity: 0.4;
            }
            th.asc::after {
                border-bottom-width: 0;
                border-top-color: #000000;
                opacity: 1;
            }
            th.desc::after {
                border-top-width: 0;
                border-bottom-color: #000000;
                opacity: 1;
            }
            th:hover::after {
                opacity: 1;
            }
            th:hover {
                background-color: rgba(0, 35, 88, 0.749);
            }
            

            th {
                background-color: rgba(0, 35, 88, 0.749); /* Change the background color to blue */
                color: white; /* Change the text color to white */
            }

             /* Show clipped text on hover */
            td:hover {
            overflow: visible;
            white-space: normal;
            word-wrap: break-word;
            }
          


            h1 {
                text-align: center;
                color: #626060;;
                font-weight: bold;
                margin: 50px;
            }
            p{
                font-family:Tahoma, Geneva, Verdana, sans-serif;
                text-align: center;
                text-justify: inter-word;
                font-size: 20px;
                
                padding-right: 100px;
                
                padding-left:100px;
            }


            .btn {
                display: flex;
                justify-content: center;
                align-items: center;
                width: 200px;
                height: 50px;
                background-color: #4CAF50;
                color: white;
                font-size: 20px;
                text-align: center;
                text-decoration: none;
                border-radius: 25px;
                margin: 50px auto;
            }

            .text_link {
                color: red;
            }
        </style>
    """
   

    table =  "<p>This page is the output of the <b>Design module</b> of the therapeutic proteins with single point mutations of the query proteins. The table below provides the details of the proteins generated from the query protein sequence with first column displaying the Mutated sequence, second column is for the motif inserted in the sequence with single point mutation(insertion, deletion, substitution or translocation), the third column is providing the score given by the machine learning algorithm, as per the prediction model, the fourth column gives the motif score, the fifth column gives the hybrid score followed by the sixth column, providing the Prediction, whether the protein is therapeutic or non-therapeutic.</p>"
    table+= "<br><br>"

    table += "<center>Job ID: <b>{}</b>. For more information, click <a href='help.php'>Help</a>. To download results as a CSV file: <a href='/dis.php?file=/tmp/thppred/thppred_{}/results_{}.csv' class='text_link' >Click Here</a></center>".format(job_id,job_id,job_id)

    table += "<table id='myTable'>"
    # add header row
    table += "<tr>"
    for col in df.columns:
        table += "<th onclick='sortTable(" + str(df.columns.get_loc(col)) + ", event)'>" + str(col) + "</th>"
    table += "</tr>"
    # add data rows
    for row in df.itertuples(index=False):
        table += "<tr>"
        for val in row:
            
            if isinstance(val, (int, float)):
                val = float(val)
                table += "<td>" + str(round(val, 2)) + "</td>"
            else:
                table += "<td>" + str(val) + "</td>"
        table += "</tr>"
    table += "</table>"

    script = """
        <script>
            var ascending = true;

            function sortTable(columnIndex, event) {
                var table, rows, switching, i, x, y, shouldSwitch;
                table = document.getElementById("myTable");
                switching = true;
                while (switching) {
                    switching = false;
                    rows = table.rows;
                    for (i = 1; i < (rows.length - 1); i++) {
                        shouldSwitch = false;
                        x = rows[i].getElementsByTagName("TD")[columnIndex];
                        y = rows[i + 1].getElementsByTagName("TD")[columnIndex];
                        if (ascending) {
                            if (x.innerHTML.toLowerCase() > y.innerHTML.toLowerCase()) {
                                shouldSwitch = true;
                                break;
                            }
                        } else {
                            if (x.innerHTML.toLowerCase() < y.innerHTML.toLowerCase()) {
                                shouldSwitch = true;
                                break;
                            }
                        }
                    }
                    if (shouldSwitch) {
                        rows[i].parentNode.insertBefore(rows[i + 1], rows[i]);
                        switching = true;
                    }
                }
                ascending = !ascending;
                event.currentTarget.classList.toggle("asc", ascending);
                event.currentTarget.classList.toggle("desc", !ascending);
            }
        </script>
    """

    return styles + script + table



def main():
    print('##############################################################################')
    print('# The program ThpPred is developed for predicting therapeutic and non therapeutic  #')
    print("# protein/peptides from their primary sequence, developed by Prof G. P. S. Raghava's group. #")
    print('# ############################################################################')
    # Default values
    fasta_file = "example.fasta"
    m = 2
    thr = 0.5

    # Check if command-line arguments are provided
    if len(sys.argv) > 1:
        # Get the filename argument
        fasta_file = sys.argv[1]
        x = sys.argv[1]
        x = str(x)

        # Get the integer argument
        if len(sys.argv) > 2:
            try:
                m = int(sys.argv[2])
                if m not in [1, 2,5,6]:
                    print("Invalid model number. Using default value.")
                    m = 2
            except ValueError:
                print("Invalid model number. Using default value.")

        # Get the float argument
        if len(sys.argv) > 3:
            try:
                thr = float(sys.argv[3])
            except ValueError:
                print("Invalid threshold value. Using default value.")

    # Process the arguments
    print('Summary of Parameters:')
    print('Input File: {}; Model: {}; Threshold: {}'.format(fasta_file, m, thr))
    
    

    # Rename the file
    if "thppred" in fasta_file:
        new_file_name = fasta_file
        new_file_path = os.path.join(os.path.dirname(fasta_file), new_file_name)
        if fasta_file != "example.fasta":
            os.rename(fasta_file, new_file_path)
            fasta_file = new_file_path

    select_model = m
    
    if select_model == 1:
        print("Running Base Model...")
        table = main_model(fasta_file,thr)
        #table.to_csv('base_model.csv', index=False)
        print(table)
        dftable = dataframe_to_html_table(table)
    elif select_model ==2:
        print("Running hybrid model...")
        table = motif_model(fasta_file,thr)
        #table.to_csv('motif_model.csv', index=False)
        print(table)
        dftable = dataframe_to_html_table(table)

    elif select_model ==5:
        print("Running motif...")
        table = motif(fasta_file)
        #table.to_csv('only_motif_model.csv', index=False)
        print(table)
        dftable = motif_table(table)
    elif select_model ==6:
        print("Running design...")
        table = design(x)
        #table.to_csv('only_motif_model.csv', index=False)
        print(table)
        dftable = design_table(table)
    else: 
        print("Invalid Input; valid inputs are: 1 and 2")
        sys.exit()  # Exit the program

    filename = 'results_{}.csv'.format(job_id)
    
    

    csv_path = dir +'/'+ filename
    if os.path.exists(csv_path):
        os.remove(csv_path)
    table.to_csv(csv_path, index=False)
    

    if os.path.exists(dir+'/'+'output.html'):
       os.remove(dir+'/'+'output.html')
    with open(dir+'/'+'output.html', 'w') as f:
        f.write(dftable)

    if os.path.exists(dir+'/'+'pfeature_result_{}.csv'.format(job_id)):
        os.remove(dir+'/'+'pfeature_result_{}.csv'.format(job_id))
    if os.path.exists(dir+'/'+'extracted_columns_{}.csv'.format(job_id)):
        os.remove(dir+'/'+'extracted_columns_{}.csv'.format(job_id))

    # Delete files with hyphens in their names
    files_to_delete = glob.glob(dir+'/'+"*-*")  # Change the pattern to match the files you want to delete

    for file_name in files_to_delete:
        os.remove(file_name) 


        
######################################################
if __name__ == "__main__":
    main()
    
    print('\n======= Thanks for using ThpPred. =====\n\n')
    print('Please cite: ThpPred\n')
